*******************************************************
* Replication code for the empirical example in       *
* "Multilevel Models for the Analysis of Comparative  *
* Survey Data: Common Problems and Some Solutions"    *
*                                                     *
* Authors: Alexander W. Schmidt-Catran,               *
*          Malcom Fairbrother                         *
*          Hans-Jürgen Andreß                         *
* Date:    April 6th, 2019                            *         *
*******************************************************


clear
version 14.1
set more off
capture log close
log using ESS_example_icsr.log, replace




***************************************
* Import Country-level Data
***************************************

import excel OECDdata.xlsx, firstrow
save OECDdata.dta, replace


***************************************
* Load Survey Data and Merge with 
* Country-level Data
***************************************

use ESS_example_icsr.dta
encode cntry, gen(Country)
recode essround 1 = 2002 2 = 2004 3 = 2006 4 = 2008 5 = 2010 6 = 2012 7 = 2014, gen(Year) 			

merge m:1 Country Year using OECDdata
	tab Country if _merge == 1													// which countries are lost due to missing country-level (OECD) data?
	keep if _merge == 3




***************************************
* Prepare variables and data for the 
* analysis
***************************************

recode gndr 2=0, gen(male)

gen attend = (rlgatnd*-1)+7														// inverse the coding of church attendance

egen CountryYears = group(Country Year)											// generate indicator variable for country-year combinations


* reduce data set to final sample
reg attend socspend gdppc  domicil eduyrs hincfel male agea i.Year				// this is not a useful regression, it's just used to mark sample faster than mixed would do
keep if e(sample)

* aggregate individual-level variables as context variables and build country-year-level means of dependent variable
foreach var of varlist eduyrs domicil attend  { 	
	bys Country Year: egen `var'_C = mean(`var')
	}
	
* build country-mean and demeaned variables and country-year means
foreach var of varlist socspend gdppc eduyrs_C domicil_C attend {
	bys Country: egen `var'_m = mean(`var')
	gen `var'_dm = `var'-`var'_m
	}
	
	
***************************************
* Descriptive Analysis
***************************************

* Describe sample (Table 4 in the paper)
tab Country Year

* Descriptive statistics of all variables (Table OA1 in the online appendix)
sum attend domicil eduyrs hincfel male agea socspend gdppc eduyrs_C domicil_C ///
    socspend_m gdppc_m eduyrs_C_m domicil_C_m socspend_dm gdppc_dm eduyrs_C_dm domicil_C_dm
	
* Correlations of all variables  (Table OA2 in the online appendix)
corr attend domicil eduyrs hincfel male agea socspend gdppc eduyrs_C domicil_C ///
     socspend_m gdppc_m eduyrs_C_m domicil_C_m socspend_dm gdppc_dm eduyrs_C_dm domicil_C_dm

	 
* Graphical presentation of between relationship with delete-one, complete sample (Figure 2 in the paper)
preserve																		// run code from here to restore command below in a single run
bys Country: gen help = _n
	keep if help == 1
gen labpos = 3																	// labpos is used to position the labels in the figure
replace labpos = 6 if Country == 18
replace labpos = 9 if Country == 22
replace labpos = 6 if Country == 10
replace labpos = 8 if Country == 7
replace labpos = 10 if Country == 13
replace labpos = 7 if Country == 23
replace labpos = 6 if Country == 12
replace labpos = 7 if Country == 2	
twoway ///
   lfit attend_C socspend_m if Country !=1, lc(gs12) lp(solid) ///
|| lfit attend_C socspend_m if Country != 2, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=4, lc(gs12) lp(solid)    ///
|| lfit attend_C socspend_m if Country !=6, lc(gs12) lp(solid)    ///
|| lfit attend_C socspend_m if Country !=7, lc(gs12) lp(solid)    ///
|| lfit attend_C socspend_m if Country !=8, lc(gs12) lp(solid)    ///
|| lfit attend_C socspend_m if Country !=9, lc(gs12) lp(solid)    ///
|| lfit attend_C socspend_m if Country !=10, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=11, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=12, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=13, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=14, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=16, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=17, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=18, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=19, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=20, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=22, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=23, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=24, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=25, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=26, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=28, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=29, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=30, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend_m if Country !=31, lc(gs12) lp(solid)  ///
|| lfit attend_m socspend_m, lw(medium) lc(gs1) lp(solid) || scatter attend_m socspend_m, mlabels(Country) msize(large) ms(o) ///
mlabv(labpos) ///
|| , legend(off) ytitle(Church attendance) xtitle(Social Spending (% of GDP))

	* Identify strongest and weakest correlation and calculate average correlation
	local min = -1
	local max = 0
	levelsof Country, local(c)
	foreach num of numlist `c' {
		qui corr attend_m socspend_m if Country != `num'
		dis "Country `num':" _col(13) r(rho)
		local rho = r(rho)
		if `rho' > `min' { 
			local min = `rho' 
			}
		if `rho' < `max' { 
			local max = `rho' 
			}
	}
	dis `min'
	dis `max'
	corr attend_m socspend_m

restore	



* Graphical presentation of bivariate within- and between relationships, full sample (Figure 3 in the paper)
preserve																		// run code from here to restore command below in a single run
bys Country Year: gen help = _n
	keep if help == 1
gen labpos = 6																	// labpos is used to position the labels in the figure
replace labpos = 5 if Country == 10
replace labpos = 7 if Country == 22
replace labpos = 1 if Country == 16
replace labpos = 11 if Country == 13
replace labpos = 12 if Country == 24
replace labpos = 1 if Country == 11
replace labpos = 10 if Country == 7
replace labpos = 12 if Country == 6
replace labpos = 12 if Country == 8
replace labpos = 10 if Country == 28
twoway lfit attend_C socspend if Country ==1, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country == 2, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==4, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country ==6, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country ==7, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country ==8, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country ==9, lc(gs12) lp(solid)   ///
|| lfit attend_C socspend if Country ==10, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==11, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==12, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==13, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==14, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==16, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==17, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==18, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==19, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==20, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==22, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==23, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==24, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==25, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==26, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==28, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==29, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==30, lc(gs12) lp(solid)  ///
|| lfit attend_C socspend if Country ==31, lc(gs12) lp(solid)  ///
|| scatter attend_m socspend_m, mlabels(Country) msize(large) mlabv(labpos) ms(o) || lfit attend_m socspend_m, lw(medium) lc(gs1)  ///
, legend(off) ytitle(Church attendance) xtitle(Social Spending (% of GDP))
restore	
	
	
***************************************
* Multivariate Analysis
***************************************

* Analyzing a single cross-section (ESS 2014--the newest wave at the time of writing)

	* Notes: After some of the following estimations, we store locals that are used to calculate 
	*        reductions of the vriance components. This will only work if you run the complete code
	*        from here to Table 1
	

	* Empty model
	mixed attend if Year == 2014 || Country: , reml								// with restricted maximum likelihood, should provide better estimates of variance components with small N
		est store M0
		* Calculate ICC
		dis exp(_b[lns1_1_1:_cons])^2/(exp(_b[lns1_1_1:_cons])^2+exp(_b[lnsig_e:_cons])^2)	// by hand
		estat icc																// using build in stata command
		local m0_cv = exp(_b[lns1_1_1:_cons])^2									// store country-level variance component for later use
		local m0_iv = exp(_b[lnsig_e:_cons])^2
		* Compare to results from full ML
		mixed attend if Year == 2014 || Country: , mle							// with full maximum likelihood (not presented in table but references in the text)
			estat icc

			
	* Add individual level variables
	mixed attend domicil eduyrs hincfel male agea if Year == 2014 || Country: , reml		// with restricted maximum likelihood, should provide better estimates of variance components
		est store M1
		* Calculate reduction of variance
		dis 1-(exp(_b[lns1_1_1:_cons])^2/`m0_cv')								// at country level
		dis 1-(exp(_b[lnsig_e:_cons])^2/`m0_iv')								// at individual level
		local m0_cv = exp(_b[lns1_1_1:_cons])^2									// store country-level variance component for later use
		local m0_iv = exp(_b[lnsig_e:_cons])^2
		
	* Add country-level variables
	mixed attend domicil eduyrs hincfel male agea socspend gdppc domicil_C eduyrs_C if Year == 2014 || Country: , reml		// with restricted maximum likelihood, should provide better estimates of variance components
		est store M2
		* Calculate reduction of variance
		dis 1-(exp(_b[lns1_1_1:_cons])^2/`m0_cv')
		dis 1-(exp(_b[lnsig_e:_cons])^2/`m0_iv')

	* Table 1 in the paper (note that estimates table does show transformed variance components,
	*                       Table 1 in the paper presents the original metric, as in the output of mixed)
	est tab M0 M1 M2, star 

	
* Analyzing comparative longitudinal survey data

	* Standard model without decomposition 
	mixed attend socspend gdppc eduyrs_C domicil_C domicil eduyrs hincfel male agea i.Year || Country: || Year: , reml var	
		est store M3
	
	* Incorrect two-level model with country-level variables that vary over time
	mixed attend socspend gdppc eduyrs_C domicil_C domicil eduyrs hincfel male agea i.Year || Country:  , reml var	
		est store M4

	* Correct two level model with country-level variables that are averaged across years
	mixed attend socspend_m gdppc_m eduyrs_C_m domicil_C_m domicil eduyrs hincfel male agea i.Year || Country:  , reml var	
		est store M5
	
	* Table 2 in the paper
	est tab M3 M4 M5, b(%9.4f) t(%9.2f)	
	est tab M3 M4 M5, star	

	
	* Model of within and between effects
	mixed attend socspend_m socspend_dm gdppc_m gdppc_dm eduyrs_C_m eduyrs_C_dm domicil_C_m domicil_C_dm domicil eduyrs hincfel male agea  i.Year  || Country: || Year: , reml var	
		est store M6


	
	
* Outlier Analysis (CooksD and DFBETAs)

	* Run models M3 and M6, leaving each country out one at a time
	levelsof Country, local(c)
	foreach num of numlist `c' {
		mixed attend socspend gdppc eduyrs_C domicil_C domicil eduyrs hincfel male agea i.Year if Country != `num' || Country: || Year: , reml var	
			est store M3_`num'
		mixed attend socspend_m socspend_dm gdppc_m gdppc_dm eduyrs_C_m eduyrs_C_dm domicil_C_m domicil_C_dm domicil eduyrs hincfel male agea  i.Year if Country != `num'  || Country: || Year: , reml var	
			est store M6_`num'
	}

	
	* CooksD fixed part (Table 5 in the paper)

	foreach m in M3 M6 {				// run loop over standard and within-between model 
		qui {
		est restore `m' 
		mat B = e(b)					// get coefficient vector			
		local eqn = e(depvar)+":"		// get equation name
		local nfc = e(k_f)				// get number of fixed coefficients
		mat B = B[1,"`eqn'"]			// reduce coefficient vector to fixed part
		mat N = e(N_g)					// get number of (highest-level) cluster (numer of countries)
		local nc = N[1,1]				// store as local
		local co = 4/`nc'				// calculate cut-off value for CooksD

		levelsof Country, local(c)

		foreach num of numlist `c' {											// calculate CooksD
			est restore `m'_`num' 	
			mat B`num' = e(b)													// get coefficient vector			
			mat B`num' = B`num'[1,"`eqn'"]										// reduce coefficient vector to fixed part
			mat V`num' = e(V)													// get variance-covariance matrix
			mat V`num' = V`num'["`eqn'","`eqn'"]								// reduce vc-matrix to fixed part
			mat C`num' = 1/`nfc' * (B-B`num') * syminv(V`num') * (B-B`num')'	// calculate CooksD 
			local cd`num' = C`num'[1,1]											// store CooksD as local
			mat drop B`num' V`num' C`num'										// get rid of matrices no longer needed
			}
		mat drop B																// get rid of matrices no longer needed
	
	noisily: dis as text "Cook's D:"
	foreach num of numlist `c' {												// display results
		local vl : value label Country											// get value label of cluster ID (Countries)
		local vll: label `vl' maxlength											// get maximum length of value labels of cluster ID
		local vll = `vll'+2														// necessary for formating the output
		local cl : label `vl' `num' 											// get label of cluster-ID at for Country = `num' 
		if `cd`num'' > `co' {													// generate local with a star if cut off value is exceeded
			local s	= "*"
			}
		else {
			local s = ""
			}
		noisily: dis _column(1) as text "`cl'" _column(`vll') as result  %9.4f `cd`num'' " `s'" 
		}
	noisily: dis as text "Legend: * cut off value (" %5.4f as result `co' as text ") exceeded"
	noisily: dis ""
	}
	}


	* DFBETAs for selected fixed effect

	* Standard Model (not presented in the paper)
	local v = "socspend"												// define variable of interest
	local m = "M3"														// define name of model
	qui {
	est restore `m' 
	local b = _b[`v']													// get coefficient of variable defined above from full model
	mat N = e(N_g)														// get number of (highest-level) cluster (numer of countries)
	local nc = N[1,1]													// store as local
	local co = 2/sqrt(`nc')												// calculate cut-off value for DFBETAs

	levelsof Country, local(c)
	foreach num of numlist `c' {
		est restore `m'_`num'
		local b_`num' = _b[`v']											// get coefficient of variable defined above from delete-one models
		local se_`num' = _se[`v']										// get standard error of variable defined above from delete-one models
		local dfbetas_`num' = (`b'-`b_`num'')/`se_`num''
		}

	noisily: dis as text "DFBETAs (`v'):"	
	foreach num of numlist `c' {										// display results
		local vl : value label Country									// get value label of cluster ID (Countries)
		local vll: label `vl' maxlength									// get maximum length of value labels of cluster ID
		local vll = `vll'+2												// necessary for formating the output
		local cl : label `vl' `num' 									// get label of cluster-ID at for Country = `num' 
		if sqrt((`dfbetas_`num'')^2) > sqrt((`co')^2) {					// generate local with a star if cut off value is exceeded
			local s	= "*"
			}
		else {
			local s = ""
			}
		noisily: dis _column(1) as text "`cl'" _column(`vll') as result  %9.4f `dfbetas_`num'' " `s'" 
		}
	noisily: dis as text "Legend: * cut off value (" %5.4f as result `co' as text ") exceeded"
	noisily: dis ""
	}



	* Within-Between Model (Table 5 in the paper)
	local v = "socspend_dm"												// define variable of interest
	local m = "M6"														// define name of model
	qui {
	est restore `m' 
	local b = _b[`v']													// get coefficient of variable defined above from full model
	mat N = e(N_g)														// get number of (highest-level) cluster (numer of countries)
	local nc = N[1,1]													// store as local
	local co = 2/sqrt(`nc')												// calculate cut-off value for DFBETAs

	levelsof Country, local(c)
	foreach num of numlist `c' {
		est restore `m'_`num'
		local b_`num' = _b[`v']											// get coefficient of variable defined above from delete-one models
		local se_`num' = _se[`v']										// get standard error of variable defined above from delete-one models
		local dfbetas_`num' = (`b'-`b_`num'')/`se_`num''
		}

	noisily: dis as text "DFBETAs (`v'):"	
	foreach num of numlist `c' {										// display results
		local vl : value label Country									// get value label of cluster ID (Countries)
		local vll: label `vl' maxlength									// get maximum length of value labels of cluster ID
		local vll = `vll'+2												// necessary for formating the output
		local cl : label `vl' `num' 									// get label of cluster-ID at for Country = `num' 
		if sqrt((`dfbetas_`num'')^2) > sqrt((`co')^2) {					// generate local with a star if cut off value is exceeded
			local s	= "*"
			}
		else {
			local s = ""
			}
		noisily: dis _column(1) as text "`cl'" _column(`vll') as result  %9.4f `dfbetas_`num'' " `s'" 
		}
	noisily: dis as text "Legend: * cut off value (" %5.4f as result `co' as text ") exceeded"
	noisily: dis ""
	}



	* Standard model, excluding outliers (identified via DFBETAs), not presented in the paper
	mixed attend socspend gdppc eduyrs_C domicil_C domicil eduyrs hincfel male agea i.Year if Country != 8 & Country != 17 & Country != 25 || Country: || Year: , reml var	
		est store M3_out

	
	* Model of within and between effects, excluding outliers (identified via DFBETAs)
	mixed attend socspend_m socspend_dm gdppc_m gdppc_dm eduyrs_C_m eduyrs_C_dm domicil_C_m domicil_C_dm domicil eduyrs hincfel male agea  i.Year  if Country != 8 & Country != 17 & Country != 25  || Country: || Year: , reml var	
		est store M7
	
	* Table 3 in the paper	
	est tab M6 M7, b(%9.4f) p(%9.4f)	stats(N)
	est tab M6 M7, star

******************************
log close

